#include "cdssmap.h"
#include <stdio.h>

//---------------------------------------------------------------------------------------
CDSSMap::CDSSMap()
{

   mVerts = NULL;
   mNumVerts = 0;

   // Get a list of selected objects
   List *ol = NULL;
   GetSelectedACObjects( &ol );

   // Allocate arrays for vert maps
   for( List *po = ol; po != NULL; po = po->next )
   {
      mNumVerts += ac_object_get_num_vertices(( ACObject * )( po->data ));
   }

   if( mNumVerts > 0 )
   {
      mVerts = new CVertMap[mNumVerts];
   }
   else
   {
      list_free( &ol );
      return;
   }

   // Populate arrays for vert maps
   unsigned int vctr = 0;
   for( List *po = ol; po != NULL; po = po->next )
   {
      ACObject *o = ac_object_get_subdivision(( ACObject * )( po->data ));
      if( o == NULL )
      {
         continue;
      }
      // Add vertices
      for( List *pv = ac_object_get_vertexlist( o ); pv != NULL; pv = pv->next )
      {
         Vertex *vs = ( Vertex * )( pv->data );
         Vertex *vo = ac_vertex_get_proxy( vs );
         if( vo != NULL )
         {
            mVerts[vctr].original = vo;
            mVerts[vctr].subdivided = vs;
            ++vctr;
         }
      }
   }

   // Sort arrays by the original vert
   QSortVertMap( 0, mNumVerts - 1 );

   list_free( &ol );

}
//---------------------------------------------------------------------------------------
CDSSMap::~CDSSMap()
{
   Clear();
}
//---------------------------------------------------------------------------------------
void CDSSMap::Clear()
{
   if( mVerts != NULL )
   {
      delete [] mVerts;
      mVerts = NULL;
   }
   mNumVerts = 0;
}
//---------------------------------------------------------------------------------------
void CDSSMap::QSortVertMap( int left, int right )
{

   int i = left;
   int j = right;
   CVertMap x, y;
   x.CopyFrom( &( mVerts[( left + right ) / 2] ));

   do
   {
      while( mVerts[i].original < x.original && i < right )
      {
         ++i;
      }
      while( x.original < mVerts[j].original && j > left )
      {
         --j;
      }
      if( i <= j )
      {
         y.CopyFrom( &( mVerts[i] ));
         mVerts[i].CopyFrom( &( mVerts[j] ));
         mVerts[j].CopyFrom( &y );
         ++i;
         --j;
      }
   } while( i <= j );

   if( left < j )
   {
      QSortVertMap( left, j );
   }
   if( i < right )
   {
      QSortVertMap( i, right );
   }

}
//---------------------------------------------------------------------------------------
void CDSSMap::ListAddObjectWithSSRecurse( List **out, ACObject *o )
{
   if( ac_object_get_vertexlist( o ) != NULL && ac_object_get_subdivision( o ) != NULL )
   {
      list_add_item_head( out, o );
   }
   for( List *po = ac_object_get_childrenlist( o ); po != NULL; po = po->next )
   {
      ListAddObjectWithSSRecurse( out, ( ACObject * )( po->data ));
   }
}
//---------------------------------------------------------------------------------------
void CDSSMap::GetSelectedACObjects( List **out )
{

   List *l = NULL;

   switch( ac_get_select_mode() )
   {
   case SELECT_VERTEX:
      l = ac_selection_get_vertices();
      for( List *pi = l; pi != NULL; pi = pi->next )
      {
         list_add_item_unique( out, object_of_vertex(( Vertex * )( pi->data )));
      }
      break;
   case SELECT_SURFACE:
      l = ac_selection_get_surfaces();
      for( List *pi = l; pi != NULL; pi = pi->next )
      {
         list_add_item_unique( out, object_of_surface(( Surface * )( pi->data )));
      }
      break;
   default:
      l = ac_selection_get_objects();
      for( List *pi = l; pi != NULL; pi = pi->next )
      {
         ListAddObjectWithSSRecurse( out, ( ACObject * )( pi->data ));
      }
      break;
   }

   list_free( &l );

}
//---------------------------------------------------------------------------------------
Vertex *CDSSMap::GetSSVertex( Vertex *original )
{
   int low = 0;
   int high = mNumVerts - 1;
   int mid;
   while( low <= high )
   {
      mid = ( low + high ) / 2;
      if( original < mVerts[mid].original )
      {
         high = mid - 1;
      }
      else if( original > mVerts[mid].original )
      {
         low = mid + 1;
      }
      else
      {
         return mVerts[mid].subdivided;
      }
   }
   return original;
}
//---------------------------------------------------------------------------------------
